@@ -277,40 +277,26 @@ func nodeType(interp *Interpreter, sc *scope, n *node) (*itype, error) {
 			}
 		}
 
-		// Because an empty interface concrete type "mutates" as different values are
-		// assigned to it, we need to make a new itype from scratch everytime a new
-		// assignment is made, and not let different nodes (of the same variable) share the
-		// same itype. Otherwise they would overwrite each other.
-		if n.anc.kind == assignStmt && isInterface(n.anc.child[0].typ) && len(n.anc.child[0].typ.field) == 0 {
-			// TODO(mpl): do the indexes properly for multiple assignments on the same line.
-			// Also, maybe we should use nodeType to figure out dt.cat? but isn't it always
-			// gonna be an interfaceT anyway?
-			dt := new(itype)
-			dt.cat = interfaceT
-			val := new(itype)
-			val.cat = t.cat
-			dt.val = val
-			// TODO(mpl): do the indexes properly for multiple assignments on the same line.
-			// Also, maybe we should use nodeType to figure out dt.cat? but isn't it always
-			// gonna be an interfaceT anyway?
-			n.anc.child[0].typ = dt
-			// TODO(mpl): not sure yet whether we should do that last step. It doesn't seem
-			// to change anything either way though.
-			// t = dt
-			break
-		}
-
 		// If the node is to be assigned or returned, the node type is the destination type.
 		dt := t
 
 		switch a := n.anc; {
+		case a.kind == assignStmt && isEmptyInterface(a.child[0].typ):
+			// Because an empty interface concrete type "mutates" as different values are
+			// assigned to it, we need to make a new itype from scratch everytime a new
+			// assignment is made, and not let different nodes (of the same variable) share the
+			// same itype. Otherwise they would overwrite each other.
+			a.child[0].typ = &itype{cat: interfaceT, val: dt}
+
 		case a.kind == defineStmt && len(a.child) > a.nleft+a.nright:
 			if dt, err = nodeType(interp, sc, a.child[a.nleft]); err != nil {
 				return nil, err
 			}
+
 		case a.kind == returnStmt:
 			dt = sc.def.typ.ret[childPos(n)]
 		}
+
 		if isInterface(dt) {
 			dt.val = t
 		}
@@ -891,6 +877,10 @@ func (t *itype) isRecursive() bool {
 	return false
 }
 
+func (t *itype) isIndirectRecursive() bool {
+	return t.isRecursive() || t.val != nil && t.val.isIndirectRecursive()
+}
+
 // isVariadic returns true if the function type is variadic.
 // If the type is not a function or is not variadic, it will
 // return false.
@@ -1172,6 +1162,10 @@ func (t *itype) id() (res string) {
 		}
 		res += "}"
 	case valueT:
+		if isConstantValue(t.rtype) {
+			res = fixPossibleConstType(t.rtype).String()
+			break
+		}
 		res = ""
 		if t.rtype.PkgPath() != "" {
 			res += t.rtype.PkgPath() + "."
@@ -1183,6 +1177,29 @@ func (t *itype) id() (res string) {
 	return res
 }
 
+// fixPossibleConstType returns the input type if it not a constant value,
+// otherwise, it returns the default Go type corresponding to the
+// constant.Value.
+func fixPossibleConstType(t reflect.Type) (r reflect.Type) {
+	cv, ok := reflect.New(t).Elem().Interface().(constant.Value)
+	if !ok {
+		return t
+	}
+	switch cv.Kind() {
+	case constant.Bool:
+		r = reflect.TypeOf(true)
+	case constant.Int:
+		r = reflect.TypeOf(0)
+	case constant.String:
+		r = reflect.TypeOf("")
+	case constant.Float:
+		r = reflect.TypeOf(float64(0))
+	case constant.Complex:
+		r = reflect.TypeOf(complex128(0))
+	}
+	return r
+}
+
 // zero instantiates and return a zero value object for the given type during execution.
 func (t *itype) zero() (v reflect.Value, err error) {
 	if t, err = t.finalize(); err != nil {
@@ -1530,6 +1547,8 @@ func (t *itype) frameType() (r reflect.Type) {
 			break
 		}
 		r = reflect.TypeOf((*valueInterface)(nil)).Elem()
+	case mapT:
+		r = reflect.MapOf(t.key.frameType(), t.val.frameType())
 	case ptrT:
 		r = reflect.PtrTo(t.val.frameType())
 	default:
@@ -1704,7 +1723,7 @@ func isInterfaceSrc(t *itype) bool {
 }
 
 func isInterfaceBin(t *itype) bool {
-	return t.cat == valueT && t.rtype.Kind() == reflect.Interface
+	return t.cat == valueT && t.rtype.Kind() == reflect.Interface || t.cat == errorT
 }
 
 func isInterface(t *itype) bool {
